import 'dart:async';
import 'package:flutter/services.dart';
import 'dart:convert' show JsonEncoder, jsonEncode;

import 'smartlook_helper.dart';

enum SmartlookNavigationEventType { enter, exit }
enum SmartlookRenderingMode { native, no_rendering, wireframe }
enum EventTrackingMode {
  FULL_TRACKING,
  IGNORE_USER_INTERACTION,
  IGNORE_NAVIGATION_INTERACTION,
  IGNORE_RAGE_CLICKS,
  NO_TRACKING
}

String describeEnum(Object enumEntry) {
  final String description = enumEntry.toString();
  final int indexOfDot = description.indexOf('.');
  assert(indexOfDot != -1 && indexOfDot < description.length - 1);
  return description.substring(indexOfDot + 1);
}

abstract class IntegrationListener {
  void onSessionReady(String? dashboardSessionUrl);
  void onVisitorReady(String? dashboardVisitorUrl);
}

class SetupOptionsBuilder {
  String ApiKey;
  int Fps = 2;
  bool StartNewSession = false;
  bool StartNewSessionAndUser = false;

  SetupOptionsBuilder(this.ApiKey);

  SetupOptions build() {
    return SetupOptions._builder(this);
  }
}

class SetupOptions {
  final String ApiKey;
  final int Fps;
  final bool StartNewSession;
  final bool StartNewSessionAndUser;

  SetupOptions._builder(SetupOptionsBuilder builder)
      : ApiKey = builder.ApiKey,
        Fps = builder.Fps,
        StartNewSession = builder.StartNewSession,
        StartNewSessionAndUser = builder.StartNewSessionAndUser;

  Map<String, dynamic> toJson() => {
        'ApiKey': ApiKey,
        'Fps': Fps,
        'StartNewSession': StartNewSession,
        'StartNewSessionAndUser': StartNewSessionAndUser,
      };
}

String getPrettyJSONString(jsonObject) {
  var encoder = new JsonEncoder.withIndent("");
  return encoder.convert(jsonObject);
}

class Smartlook {
  static const MethodChannel _channel = const MethodChannel('smartlook');

  static const EventChannel _eventChannel =
      const EventChannel('smartlookEvent');

  static Future<dynamic> nativeMethodCallHandler(MethodCall methodCall) async {
    switch (methodCall.method) {
      case "obtainWireframeData":
        final wireframeData =
            await SmartlookHelperController.instance.buildWireframeTree();
        return getPrettyJSONString(wireframeData);
      default:
        return "Nothing";
    }
  }

  // SETUP
  static void setup(SetupOptions options) async {
    await _channel
        .invokeMethod('setupBridge', {"options": jsonEncode(options.toJson())});
  }

  static Future<void> setupAndStartRecording(SetupOptions options) async {
    _channel.setMethodCallHandler(nativeMethodCallHandler);
    await _channel.invokeMethod('setupAndStartRecordingBridge',
        {"options": jsonEncode(options.toJson())});
  }

  static Future<void> setUserIdentifier(String key,
      [Object? map = null]) async {
    await _channel.invokeMethod('setUserIdentifier', {"key": key, "map": map});
  }

  // START & STOP
  static Future<void> startRecording() async {
    await _channel.invokeMethod('startRecording');
  }

  static Future<void> stopRecording() async {
    await _channel.invokeMethod('stopRecording');
  }

  static Future<bool?> isRecording() async {
    bool? isRecording = await _channel.invokeMethod('isRecording');
    return isRecording;
  }

  // EVENTS
  static Future<String?> startTimedCustomEvent(String key,
      [Object? map = null]) async {
    String? eventId = await _channel
        .invokeMethod('startTimedCustomEvent', {"key": key, "map": map});
    return eventId;
  }

  static Future<void> stopTimedCustomEvent(String key,
      [Object? map = null]) async {
    await _channel
        .invokeMethod('stopTimedCustomEvent', {"key": key, "map": map});
  }

  static Future<void> cancelTimedCustomEvent(String key, String reason,
      [Object? map = null]) async {
    await _channel.invokeMethod(
        'cancelTimedCustomEvent', {"key": key, "reason": reason, "map": map});
  }

  static Future<void> trackCustomEvent(String key, [Object? map = null]) async {
    await _channel.invokeMethod('trackCustomEvent', {"key": key, "map": map});
  }

  static Future<void> trackNavigationEvent(
      String key, SmartlookNavigationEventType type) async {
    await _channel.invokeListMethod(
        "trackNavigationEvent", {"key": key, "type": type.index});
  }

  // SENSITIVE
  static Future<void> enableWebviewRecording(bool enabled) async {
    await _channel.invokeMethod('enableWebviewRecording', {"enabled": enabled});
  }

  // GLOBAL EVENT PROPERTIES
  static Future<void> setGlobalEventProperty(
      String key, String value, bool immutable) async {
    await _channel.invokeMethod('setGlobalEventProperty',
        {"key": key, "value": value, "immutable": immutable});
  }

  static Future<void> setGlobalEventProperties(
      Object map, bool immutable) async {
    await _channel.invokeMethod(
        'setGlobalEventProperties', {"map": map, "immutable": immutable});
  }

  static Future<void> removeGlobalEventProperty(String key) async {
    await _channel.invokeMethod('removeGlobalEventProperty', {"key": key});
  }

  static Future<void> removeAllGlobalEventProperties() async {
    await _channel.invokeMethod('removeAllGlobalEventProperties');
  }

  // OTHERS
  static Future<void> setReferrer(String referrer, String source) async {
    await _channel
        .invokeMethod('setReferrer', {"referrer": referrer, "source": source});
  }

  static Future<String?> getDashboardSessionUrl(
      bool withCurrentTimestamp) async {
    String? url = await _channel.invokeMethod('getDashboardSessionUrl',
        {"withCurrentTimestamp": withCurrentTimestamp});
    return url;
  }

  static Future<String?> getDashboardVisitorUrl() async {
    String? url = await _channel.invokeMethod('getDashboardVisitorUrl');
    return url;
  }

  static Future<void> enableCrashlytics(bool enabled) async {
    await _channel.invokeMethod('enableCrashlytics', {"enabled": enabled});
  }

  static Future<void> resetSession(bool resetUser) async {
    await _channel.invokeMethod('resetSession', {"resetUser": resetUser});
  }

  static Future<void> setRenderingMode(
      SmartlookRenderingMode renderingMode) async {
    await _channel.invokeMethod(
        'setRenderingMode', {"renderingMode": renderingMode.index});
  }

  static Future<void> setEventTrackingMode(
      EventTrackingMode eventTrackingMode) async {
    await _channel.invokeMethod('setEventTrackingMode',
        {"eventTrackingMode": describeEnum(eventTrackingMode)});
  }

  static Future<void> setEventTrackingModes(
      List<EventTrackingMode> eventTrackingModes) async {
    List<String> trackingModes =
        eventTrackingModes.map((mode) => describeEnum(mode)).toList();
    await _channel.invokeMethod(
        'setEventTrackingModes', {"eventTrackingModes": trackingModes});
  }

  static Future<void> registerIntegrationListener(
      IntegrationListener integrationListener) async {
    await _eventChannel.receiveBroadcastStream().listen((event) {
      //print(event);
      if (event.contains('visitor')) {
        integrationListener.onVisitorReady(event);
      } else {
        integrationListener.onSessionReady(event);
      }
    });
  }
}
